@@ -1,13 +1,7 @@
-# -*- coding: utf-8 -*-
 #
 #  This file is part of Sequana software
 #
-#  Copyright (c) 2016 - Sequana Development Team
-#
-#  File author(s):
-#      Thomas Cokelaer <thomas.cokelaer@pasteur.fr>
-#      Dimitri Desvillechabrol <dimitri.desvillechabrol@pasteur.fr>, 
-#          <d.desvillechabrol@gmail.com>
+#  Copyright (c) 2016-2021 - Sequana Development Team
 #
 #  Distributed under the terms of the 3-clause BSD license.
 #  The full license is in the LICENSE file, distributed with this software.
@@ -16,17 +10,19 @@
 #  documentation: http://sequana.readthedocs.io
 #
 ##############################################################################
-"""Utilities to manipulate FASTQ and Reads"""
+"""Utilities to manipulate FastA files"""
 import os
 from pysam import FastxFile
 from easydev import Progress
+import textwrap
+
+from sequana.stats import N50, L50
 
 import colorlog
+
 logger = colorlog.getLogger(__name__)
 
 
-
-
 __all__ = ["FastA"]
 
 
@@ -37,19 +33,26 @@ def is_fasta(filename):
             assert line.startswith(">")
             line = fin.readline()
             return True
-        except: #pragma: no cover
+        except:  # pragma: no cover
             return False
 
 
 # cannot inherit from FastxFile (no object in the API ?)
 class FastA(object):
-    """Class to handle FastA files. Cannot be compressed
+    """Class to handle FastA files
 
 
+    ::
+
+        from sequana import FastA
+        f = FastA("test.fa")
+        read = next(f)
+
+        names = f.names
+
     """
+
     def __init__(self, filename, verbose=False):
-        if filename.endswith(".gz"): #pragma: no cover
-            raise ValueError("Must be decompressed.")
         self._fasta = FastxFile(filename)
         self.filename = filename
         self._N = None
@@ -57,15 +60,15 @@ class FastA(object):
     def __iter__(self):
         return self
 
-    def __next__(self): # python 3
+    def __next__(self):  # python 3
         return self.next()
 
-    def next(self): # python 2
+    def next(self):  # python 2
         # reads 4 lines
         try:
             d = next(self._fasta)
             return d
-        except KeyboardInterrupt:  #pragma: no cover
+        except KeyboardInterrupt:  # pragma: no cover
             # This should allow developers to break a loop that takes too long
             # through the reads to run forever
             self._fasta.close()
@@ -77,53 +80,56 @@ class FastA(object):
 
     def __len__(self):
         if self._N is None:
-            logger.info("Reading input fasta file...please wait") 
+            logger.info("Reading input fasta file...please wait")
             self._N = len([x for x in FastxFile(self.filename)])
         return self._N
 
     def _get_names(self):
         return [this.name for this in self]
+
     names = property(_get_names)
 
     def _get_sequences(self):
         return [this.sequence for this in self]
+
     sequences = property(_get_sequences)
 
     def _get_comment(self):
         return [this.comment for this in self]
+
     comments = property(_get_comment)
 
     def _get_lengths(self):
         return [len(this.sequence) for this in self]
+
     lengths = property(_get_lengths)
 
     def get_lengths_as_dict(self):
+        """Return dictionary with sequence names and lengths as keys/values"""
         return dict(zip(self.names, self.lengths))
 
     def format_contigs_denovo(self, output_file, len_min=500):
-        """Replace NODE with the project name and remove contigs with a length 
-        lower than len_min.
+        """Remove contigs with sequence length below specific threshold.
 
         :param str output_file: output file name.
         :param int len_min: minimal length of contigs.
 
-        Example:
+        Example::
 
             from sequana import FastA
 
             contigs = FastA("denovo_assembly.fasta")
-            contigs.format_contigs_denovo("path/to/file.fasta", len_min=500)
+            contigs.format_contigs_denovo("output.fasta", len_min=500)
 
-        Results are stored in "path/to/file.fasta".
         """
         # catch basename of file without extension
         project = os.path.basename(output_file).split(".")[0]
         # check if directory exist
         output_dir = os.path.dirname(output_file)
         try:
-            if not os.path.exists(output_dir): #pragma: no cover
+            if not os.path.exists(output_dir):  # pragma: no cover
                 os.makedirs(output_dir)
-        except FileNotFoundError: #pragma: no cover
+        except FileNotFoundError:  # pragma: no cover
             pass
 
         n = 1
@@ -132,14 +138,21 @@ class FastA(object):
                 if len(contigs.sequence) < len_min:
                     break
                 name = ">{}_{} {}\n".format(project, n, contigs.name)
-                sequence = "\n".join([contigs.sequence[i:min(i+80, 
-                    len(contigs.sequence))] for i in range(0, 
-                    len(contigs.sequence), 80)]) + "\n"
+                sequence = (
+                    "\n".join(
+                        [
+                            contigs.sequence[i : min(i + 80, len(contigs.sequence))]
+                            for i in range(0, len(contigs.sequence), 80)
+                        ]
+                    )
+                    + "\n"
+                )
                 fp.write(name + sequence)
                 n += 1
 
     def filter(self, output_filename, names_to_keep=None, names_to_exclude=None):
-        if names_to_exclude is None and names_to_keep is None: #pragma: no cover
+        """save FastA excluding or including specific sequences"""
+        if names_to_exclude is None and names_to_keep is None:  # pragma: no cover
             logger.warning("No ids provided")
             return
 
@@ -180,9 +193,10 @@ class FastA(object):
         :param str output_filename:
         """
         import numpy as np
+
         thisN = len(self)
         if isinstance(N, int):
-            if N > thisN: 
+            if N > thisN:
                 N = thisN
             # create random set of reads to pick up
             cherries = list(range(thisN))
@@ -194,23 +208,24 @@ class FastA(object):
         elif isinstance(N, list):
             cherries = set(N)
         fasta = FastxFile(self.filename)
-        pb = Progress(thisN) # since we scan the entire file
+        pb = Progress(thisN)  # since we scan the entire file
         with open(output_filename, "w") as fh:
             for i, read in enumerate(fasta):
                 if i in cherries:
                     fh.write(read.__str__() + "\n")
                 else:
                     pass
-                pb.animate(i+1)
+                pb.animate(i + 1)
         return cherries
 
     def get_stats(self):
+        """Return a dictionary with basic statistics"""
         from pylab import mean
+
         stats = {}
         stats["N"] = len(self.sequences)
         stats["mean_length"] = mean(self.lengths)
         stats["total_length"] = sum(self.lengths)
-        from sequana.stats import N50, L50
         stats["N50"] = N50(self.lengths)
         stats["L50"] = L50(self.lengths)
         stats["min_length"] = min(self.lengths)
@@ -218,7 +233,14 @@ class FastA(object):
         return stats
 
     def summary(self, max_contigs=-1):
+        """returns summary and print information on the stdout
+
+        This method is used when calling sequana standalone ::
+
+            sequana summary test.fasta
+        """
         from pylab import mean, argmax
+
         # used by sequana summary fasta
         summary = {"number_of_contigs": len(self.sequences)}
         summary["total_contigs_length"] = sum(self.lengths)
@@ -230,13 +252,13 @@ class FastA(object):
         positions = list(range(len(lengths)))
         stats = self.get_stats()
         print("#sample_name: {}".format(self.filename))
-        print("#total length: {}".format(stats['total_length']))
-        print("#N50: {}".format(stats['N50']))
-        print("#Ncontig: {}".format(stats['N']))
-        print("#L50: {}".format(stats['L50']))
-        print("#max_contig_length: {}".format(stats['max_length']))
-        print("#min_contig_length: {}".format(stats['min_length']))
-        print("#mean_contig_length: {}".format(stats['mean_length']))
+        print("#total length: {}".format(stats["total_length"]))
+        print("#N50: {}".format(stats["N50"]))
+        print("#Ncontig: {}".format(stats["N"]))
+        print("#L50: {}".format(stats["L50"]))
+        print("#max_contig_length: {}".format(stats["max_length"]))
+        print("#min_contig_length: {}".format(stats["min_length"]))
+        print("#mean_contig_length: {}".format(stats["mean_length"]))
 
         print("contig name,length,count A,C,G,T,N")
         if max_contigs == -1:
@@ -248,37 +270,52 @@ class FastA(object):
             position = positions.pop(index)
             sequence = self.sequences[position]
             name = self.names[position]
-            print("{},{},{},{},{},{},{}".format(name, length, sequence.count('A'), sequence.count('C'),
-                sequence.count('G'), sequence.count('T'), sequence.count('N')))
-            
+            print(
+                "{},{},{},{},{},{},{}".format(
+                    name,
+                    length,
+                    sequence.count("A"),
+                    sequence.count("C"),
+                    sequence.count("G"),
+                    sequence.count("T"),
+                    sequence.count("N"),
+                )
+            )
+
     def GC_content_sequence(self, sequence):
-        GC = sequence.count('G') + sequence.count('g')
-        GC += sequence.count('C') + sequence.count('c')
-        return GC /  len(sequence) * 100
+        """Return GC content in percentage of a sequence"""
+        GC = sequence.count("G") + sequence.count("g")
+        GC += sequence.count("C") + sequence.count("c")
+        return GC / len(sequence) * 100
 
     def GC_content(self):
+        """Return GC content in percentage of all sequences found in the FastA file"""
         lengths = sum(self.lengths)
         GC = 0
         for seq in self.sequences:
-            GC += seq.count('G') + seq.count('g')
-            GC += seq.count('C') + seq.count('c')
+            GC += seq.count("G") + seq.count("g")
+            GC += seq.count("C") + seq.count("c")
         return GC / lengths * 100
-        
 
     def reverse_and_save(self, filename):
+        """Reverse sequences and save in a file"""
         with open(filename, "w") as fout:
             for read in self:
-                fout.write(">{}\t{}\n{}\n".format(read.name, read.comment,
-                    read.sequence[::-1]))
+                fout.write(
+                    ">{}\t{}\n{}\n".format(read.name, read.comment, read.sequence[::-1])
+                )
 
     def save_ctg_to_fasta(self, ctgname, outname, max_length=-1):
+        """Select a contig and save in a file"""
         index = self.names.index(ctgname)
         with open("{}.fa".format(outname), "w") as fout:
 
             if max_length == -1:
                 fout.write(">{}\n{}".format(outname, self.sequences[index]))
             else:
-                fout.write(">{}\n{}".format(outname, self.sequences[index][0:max_length]))
+                fout.write(
+                    ">{}\n{}".format(outname, self.sequences[index][0:max_length])
+                )
 
     def to_fasta(self, outfile, width=80):
         """Save the input FastA file into a new file
@@ -288,28 +325,26 @@ class FastA(object):
 
         """
         with open(outfile, "w") as fout:
-            for name,comment,seq in zip(self.names, self.comments, self.sequences):
-                import textwrap
+            for name, comment, seq in zip(self.names, self.comments, self.sequences):
                 seq = "\n".join(textwrap.wrap(seq, width))
                 if comment is None:
-                    fout.write(">{}\n{}\n".format(name, seq))
+                    fout.write(f">{name}\n{seq}\n")
                 else:
-                    fout.write(">{}\t{}\n{}\n".format(name, comment, seq))
+                    fout.write(f">{name}\t{comment}\n{seq}\n")
 
     def to_igv_chrom_size(self, output):
+        """Create a IGV file storing chromosomes and their sizes"""
         data = self.get_lengths_as_dict()
         with open(output, "w") as fout:
-            for k,v in data.items():
+            for k, v in data.items():
                 fout.write("{}\t{}\n".format(k, v))
 
-
-    
-
-
-
-
-
-
-
-
-
+    def save_collapsed_fasta(self, outfile, ctgname, width=80, comment=None):
+        """Concatenate all contigs and save results"""
+        with open(outfile, "w") as fout:
+            data = "".join(self.sequences)
+            seq = "\n".join(textwrap.wrap(data, width))
+            if comment is None:
+                fout.write(f">{ctgname}\n{seq}\n")
+            else:
+                fout.write(f">{ctgname}\t{comment}\n{seq}\n")
