@@ -14,11 +14,10 @@
 #  documentation: http://sequana.readthedocs.io
 #
 ##############################################################################
+import colorlog
+logger = colorlog.getLogger(__name__)
 
 
-from sequana import logger
-logger.name = __name__
-
 
 """
 There are 20 amino acids (Stored in :ref:`amino_acids`) such as alanine.
