@@ -14,13 +14,16 @@
 #  documentation: http://sequana.readthedocs.io
 #
 ##############################################################################
-
 import os
 import re
+from collections import Counter
+from functools import wraps
+
 from sequana import sequana_config_path
 from sequana.lazy import pandas as pd
-from functools import wraps
+from sequana.utils.singleton import Singleton
 from sequana.misc import wget
+
 from easydev import TempFile
 
 import colorlog
@@ -69,8 +72,7 @@ class NCBITaxonomy():
         inherited GC  flag  (1 or 0)        -- 1 if node inherits genetic code from parent
         mitochondrial genetic code id       -- see gencode.dmp file
         inherited MGC flag  (1 or 0)        -- 1 if node inherits mitochondrial gencode from parent
-        GenBank hidden flag (1 or 0)            -- 1 if name is suppressed in
-        GenBank entry lineage
+        GenBank hidden flag (1 or 0)            -- 1 if name is suppressed in GenBank entry lineage
         hidden subtree root flag (1 or 0)       -- 1 if this subtree has no sequence data yet
         comments                -- free-text comments and citations
         """
@@ -145,7 +147,7 @@ def load_taxons(f):
     return wrapper
 
 
-class Taxonomy(object):
+class Taxonomy(metaclass=Singleton):
     """This class should ease the retrieval and manipulation of Taxons
 
     There are many resources to retrieve information about a Taxon.
@@ -172,6 +174,11 @@ class Taxonomy(object):
         >>> t[9606]
         >>> t.get_lineage(9606)
 
+
+    Possible ranks are various. You may have biotype, clade, etc ub generally speaking 
+    ranks are about lineage. For a given rank, e.g. kingdom, you may have sub division such
+    as superkingdom and subkingdom. order has even more subdivisions (infra, parv, sub, super)
+
     """
     def __init__(self, filename=None, verbose=True, online=True, source="ncbi"):
         """.. rubric:: constructor
@@ -179,8 +186,8 @@ class Taxonomy(object):
         :param offline: if you do not have internet, the connction to Ensembl
             may hang for a while and fail. If so, set **offline** to True
         :param from: download taxonomy databases from ncbi
-
         """
+
         assert source in ['ncbi', 'ena']
         self.source = source
 
@@ -200,7 +207,7 @@ class Taxonomy(object):
         self._custom_db = sequana_config_path
         self._custom_db += "/taxonomy/taxonomy_custom.dat"
 
-    def _update_custom_taxonomy_bases(self, taxid):
+    def _update_custom_taxonomy_bases(self, taxid): # pragma: no cover
         """
         """
         taxid = str(taxid)
@@ -218,7 +225,7 @@ class Taxonomy(object):
                 #fout.write("GC ID : {}\n".format(data['']))
                 fout.write("SCIENTIFIC NAME : {}\n".format(data['scientificname']))
 
-    def download_taxonomic_file(self, overwrite=False):
+    def download_taxonomic_file(self, overwrite=False): #pragma: no cover
         """Loads entire flat file from EBI
 
         Do not overwrite the file by default.
@@ -311,7 +318,6 @@ class Taxonomy(object):
         if self.verbose:
             print()
            
-
     def find_taxon(self, taxid, mode="ncbi"):
         taxid = str(taxid)
         if mode == "ncbi":
@@ -321,16 +327,6 @@ class Taxonomy(object):
         else:
             res = self.ensembl.get_taxonomy_by_id(taxid)
         return res
-        """if "error" in res[taxid]:
-            print("not found in NCBI (EUtils)")
-        else:
-            data = res[taxid]
-            fout.write("ID : {}\n".format(taxid))
-            #fout.write("PARENT ID : {}\n".format(taxid))
-            fout.write("RANK : {}\n".format(data['rank']))
-            #fout.write("GC ID : {}\n".format(data['']))
-            fout.write("SCIENTIFIC NAME : {}\n".format(data['scientificname']))
-        """
 
     @load_taxons
     def fetch_by_id(self, taxon):
@@ -366,23 +362,6 @@ class Taxonomy(object):
         res = self.ensembl.get_taxonomy_by_name(name)
         return res
 
-    def on_web(self, taxon):
-        """Open UniProt page for a given taxon"""
-        # Should work for python2 and 3
-        import webbrowser
-        try:
-            from urllib.request import urlopen
-            from urllib.error import HTTPError, URLError
-        except:
-            from urllib2 import urlopen, HTTPError, URLError
-        try:
-            urlopen('http://www.uniprot.org/taxonomy/%s' % taxon)
-            webbrowser.open("http://www.uniprot.org/taxonomy/%s" % taxon)
-        except HTTPError as err:
-            print("Invalid taxon")
-        except URLError as err:
-            print(err.args)
-
     @load_taxons
     def get_lineage(self, taxon):
         """Get lineage of a taxon
@@ -459,25 +438,12 @@ class Taxonomy(object):
         children = [child['id'] for child in children]
         return children
 
-    @load_taxons
-    def get_family_tree(self, taxon):
-        """root is taxon and we return the corresponding tree"""
-        # should limit the tree size
-        # uniprot flat files has no record about children, so we would
-        # need to reconstruct the tree
-        tree = {}
-        children = self.get_children(taxon)
-        if len(children) == 0:
-            return tree
-        else:
-            return [self.get_family_tree(child) for child in children]
-
     @load_taxons
     def __getitem__(self, iden):
         return self.records[iden]
 
     @load_taxons
-    def __getitem__(self, iden):
+    def __len__(self):
         return len(self.records)
 
     def append_existing_database(self, filename):
