@@ -1,11 +1,8 @@
-# -*- coding: utf-8 -*-
 #
 #  This file is part of Sequana software
 #
-#  Copyright (c) 2016-2020 - Sequana Development Team
+#  Copyright (c) 2016-2021 - Sequana Development Team
 #
-#  File author(s):
-#      Thomas Cokelaer <thomas.cokelaer@pasteur.fr>
 #
 #  Distributed under the terms of the 3-clause BSD license.
 #  The full license is in the LICENSE file, distributed with this software.
@@ -16,13 +13,15 @@
 ##############################################################################
 
 from sequana.lazy import pylab
-import colorlog
-logger = colorlog.getLogger(__name__)
-
 from sequana.viz import clusterisation
 
+from adjustText import adjust_text
+import colorlog
 
-__all__ = ['PCA']
+logger = colorlog.getLogger(__name__)
+
+
+__all__ = ["PCA"]
 
 
 class PCA(clusterisation.Cluster):
@@ -44,11 +43,11 @@ class PCA(clusterisation.Cluster):
         p.plot(n_components=2)
 
     """
+
     def __init__(self, data, colors={}):
         super(PCA, self).__init__(data, colors)
 
-    def plot_pca_vs_max_features(self, step=100, n_components=2,
-            progress=True):
+    def plot_pca_vs_max_features(self, step=100, n_components=2, progress=True):
         """
 
         .. plot::
@@ -66,7 +65,7 @@ class PCA(clusterisation.Cluster):
             p.plot_pca_vs_max_features()
 
         """
-        assert n_components in [2,3,4]
+        assert n_components in [2, 3, 4]
         N = len(self.df)
         if step > N:
             step = N
@@ -74,32 +73,43 @@ class PCA(clusterisation.Cluster):
         # We start with at least 5 features
         X = range(10, N, step)
         from easydev import Progress
+
         pb = Progress(len(X))
         Y = []
         for i, x in enumerate(X):
             res = self.plot(n_components=n_components, max_features=x, show_plot=False)
             Y.append(res)
-            if progress: pb.animate(i+1)
+            if progress:
+                pb.animate(i + 1)
 
         sub = n_components
-        pylab.subplot(sub,1,1)
-        pylab.plot(X, [y[0]*100 for y in Y])
+        pylab.subplot(sub, 1, 1)
+        pylab.plot(X, [y[0] * 100 for y in Y])
         pylab.ylabel("PC1 (%)")
-        pylab.subplot(sub,1,2)
-        pylab.plot(X, [y[1]*100 for y in Y])
+        pylab.subplot(sub, 1, 2)
+        pylab.plot(X, [y[1] * 100 for y in Y])
         pylab.ylabel("PC2 (%)")
         if sub >= 3:
-            pylab.subplot(sub,1,3)
-            pylab.plot(X, [y[2]*100 for y in Y])
+            pylab.subplot(sub, 1, 3)
+            pylab.plot(X, [y[2] * 100 for y in Y])
             pylab.ylabel("PC3 (%)")
         if sub >= 4:
-            pylab.subplot(sub,1,4)
-            pylab.plot(X, [y[3]*100 for y in Y])
+            pylab.subplot(sub, 1, 4)
+            pylab.plot(X, [y[3] * 100 for y in Y])
             pylab.ylabel("PC4 (%)")
 
-    def plot(self, n_components=2, transform="log", switch_x=False,
-            switch_y=False, switch_z=False, colors=None,
-            max_features=500, show_plot=True):
+    def plot(
+        self,
+        n_components=2,
+        transform="log",
+        switch_x=False,
+        switch_y=False,
+        switch_z=False,
+        colors=None,
+        max_features=500,
+        show_plot=True,
+        fontsize=10,
+    ):
         """
 
         :param n_components: at number starting at 2 or a value below 1
@@ -108,7 +118,7 @@ class PCA(clusterisation.Cluster):
         :param transform: can be 'log' or 'anscombe', log is just log10. count
             with zeros, are set to 1
         """
-        assert transform in ['log', 'anscombe']
+        assert transform in ["log", "anscombe"]
 
         from sklearn.decomposition import PCA
         import numpy as np
@@ -116,7 +126,9 @@ class PCA(clusterisation.Cluster):
         pylab.clf()
         pca = PCA(n_components)
 
-        data, kept = self.scale_data(transform_method=transform, max_features=max_features)
+        data, kept = self.scale_data(
+            transform_method=transform, max_features=max_features
+        )
 
         pca.fit(data.T)
 
@@ -124,25 +136,34 @@ class PCA(clusterisation.Cluster):
         self.Xr = Xr
 
         if switch_x:
-            Xr[:,0] *= -1
+            Xr[:, 0] *= -1
         if switch_y:
-            Xr[:,1] *= -1
+            Xr[:, 1] *= -1
         if switch_z:
-            Xr[:,2] *= -1
+            Xr[:, 2] *= -1
+
+        def adjust():
+            texts = [
+                x
+                for x in pylab.gca().get_children()
+                if "text.Annotation" in str(x.__class__)
+            ]
+            adjust_text(texts)
 
         # PC1 vs PC2
         if show_plot:
             pylab.figure(1)
-            self._plot(Xr, pca=pca, pc1=0,pc2=1, colors=colors)
+            self._plot(Xr, pca=pca, pc1=0, pc2=1, colors=colors, fontsize=fontsize)
+            adjust()
 
         if len(pca.explained_variance_ratio_) >= 3:
             if show_plot:
                 pylab.figure(2)
-                self._plot(Xr, pca=pca, pc1=0,pc2=2, colors=colors)
+                self._plot(Xr, pca=pca, pc1=0, pc2=2, colors=colors, fontsize=fontsize)
+                adjust()
+
                 pylab.figure(3)
-                self._plot(Xr, pca=pca, pc1=1,pc2=2, colors=colors)
+                self._plot(Xr, pca=pca, pc1=1, pc2=2, colors=colors, fontsize=fontsize)
+                adjust()
 
         return pca.explained_variance_ratio_
-
-
-
